/*
 *
 *              Reverse MX Filter for Postfix 2
 *                    reject_bad_rmx
 *           (c) 2004 Elita rozanski@sergiusz.com
 *                    v. 2004.02.04
 *
 */

/*
MXfilter is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.
*/

#include <netinet/in.h>
#include <resolv.h>
#include <sys/socket.h>
#include <arpa/nameser.h>
#include <stdio.h>
#include <netdb.h>
#include <syslog.h>

#include "mxget.c"

// only one recipient address
#define TESTER_ONLY

int check_mx(char *sender,char *client_address,int tolerance) {
struct hostent *hh;
int i;
char *domain;
char relay[1024];

if (!index(sender,'@')) return 1; //invalid email address
domain=index(sender,'@')+1;
if (strlen(domain)<3) return 1; //domain too short

/* mx first */
if(find_mailhost(domain,client_address,tolerance)==0) return 0; //its mx!

/* a */
hh=gethostbyname(domain);
if(!hh) return 2; //dns error
for(i=0;i<256;i++) {
    if(hh->h_addr_list[i]) {
	char odp[256];
	snprintf(odp,sizeof(odp),"%d.%d.%d.%d",(unsigned char)hh->h_addr_list[i][0],
	    (unsigned char)hh->h_addr_list[i][1],
	    (unsigned char)hh->h_addr_list[i][2],
	    (tolerance==1)?0:(unsigned char)hh->h_addr_list[i][3]);
	if (strncmp(odp,client_address,strlen(odp)-tolerance)==0) {
	    return 0;
	    }
	}
    else {
	i=256;
	}
    }
/* a 'relay' */
snprintf(relay,sizeof(relay),"relay.%s",domain);
hh=gethostbyname(relay);
if(!hh) return 2; //dns error
for(i=0;i<256;i++) {
    if(hh->h_addr_list[i]) {
	char odp[256];
	snprintf(odp,sizeof(odp),"%d.%d.%d.%d",(unsigned char)hh->h_addr_list[i][0],
	    (unsigned char)hh->h_addr_list[i][1],
	    (unsigned char)hh->h_addr_list[i][2],
	    (tolerance==1)?0:(unsigned char)hh->h_addr_list[i][3]);
	if (strncmp(odp,client_address,strlen(odp)-tolerance)==0) {
	    return 0;
	    }
	}
    else {
	i=256;
	}
    }
return 1; //sorry not found :(
}

int main (void) {
#ifdef TESTER_ONLY
char tester[]="rozanski@sergiusz.com";
#endif
int log=1;

char buf[1001];
char *k;

char sender[257];
char client_address[257];
char recipient[257];

// int i=0;

memset(&sender,0,sizeof(sender));
memset(&client_address,0,sizeof(client_address));
memset(&recipient,0,sizeof(recipient));

/* input
sender=ktos@comm.pl
client_address=213.25.233.144
recipient=rozanski@sergiusz.com
*/

while((fgets(buf,1000,stdin))) {
    k=(char *)index(buf,10);
    if(k)k[0]=0;
    k=(char *)index(buf,13);
    if(k)k[0]=0;

//    printf("%d %s\n",i++,buf);

    if(strncmp(buf,"sender=",7)==0) {
	snprintf(sender,sizeof(sender),"%s",index(buf,'=')+1);
	}
    if(strncmp(buf,"client_address=",15)==0) {
	snprintf(client_address,sizeof(client_address),"%s",index(buf,'=')+1);
	}
    if(strncmp(buf,"recipient=",10)==0) {
	snprintf(recipient,sizeof(recipient),"%s",index(buf,'=')+1);
	}
    if(strlen(buf)==0) {
	int result=0;
	
	k=(char *)index(sender,'@');
	if(!k) {
	    if(log) {
		openlog("postfix/mxfilter", 0, LOG_MAIL);
		syslog(0,"%s %s %s %s\n",client_address,sender,recipient,"ERROR: bad sender");
		closelog();
		}
	    printf("action=550 Sender address rejected\n\n"); //not @ in sender?, bug in postfix?
	    return 0;
	    }

#ifdef TESTER_ONLY
	if(strcmp(recipient,tester)==0) {
	    result=check_mx(sender,client_address,1);
	    }
#else
	result=check_mx(sender,client_address,1);
#endif

	if(result==0) {
	    printf("action=dunno\n\n"); //ok MX or A record
	    }
	else {
// TODO: error in result
	    printf("action=550 Host %s not MX or A for domain %s. Check dns or use true relay host. http://rmxf.comm.pl/\n\n",
		client_address,
		index(sender,'@')+1);
	    }

	if(log) {
	    openlog("postfix/mxfilter", 0, LOG_MAIL);
	    syslog(0,"%s %s %s %s\n",client_address,sender,recipient,(result==0)?"OK":"REJECT");
	    closelog();
	    }

	return 0;
	}
    }

// strange, postfix false?
if(log) {
    openlog("postfix/mxfilter", 0, LOG_MAIL);
    syslog(0,"%s %s %s %s\n",client_address,sender,recipient,"ERROR: stdin empty");
    closelog();
    }
printf("action=dunno\n\n");
return 0;
}
